@@ -1,9 +1,6 @@
 /* Copyright 2001 by Roger S. Bivand. */
 
-#include <R.h>
-#include <Rdefines.h>
-#include <R_ext/Applic.h>
-#define ROFFSET 1
+#include "spdep.h"
 
 SEXP symtest(SEXP nb, SEXP card, SEXP verbose)
 {
@@ -14,6 +11,7 @@ SEXP symtest(SEXP nb, SEXP card, SEXP verbose)
 
 	fstop = 0;
 	for (i=0; i < n; i++) {
+            R_CheckUserInterrupt();
 	    icard = INTEGER_POINTER(card)[i];
 	    for (j=0; j<icard; j++) {
 		flag = 0;
